#version 450

in vec3 position;
in vec3 normal;
in vec2 uv;

uniform mat4 projection;
uniform mat4 view;
uniform mat4 model;

uniform ivec3 light_offset;
uniform vec3 sun_light;


out vec2 uv_out;
out vec3 normal_out;
out vec3 lighting;

layout(r11f_g11f_b10f) uniform image3D lighting_map;

void main() {
	vec4 vertex_pos = model * vec4(position, 1.);

	ivec3 light_position = ivec3(floor(-(light_offset - vertex_pos.xyz)));

	if (any(lessThan(light_position, ivec3(0))) || any(greaterThanEqual(light_position, imageSize(lighting_map).xyz))) {
		lighting = sun_light;
	} else {
		lighting = imageLoad(lighting_map, light_position).rgb;
	}

	lighting *= 0.8;
	uv_out = uv;

	normal_out = mat3(transpose(inverse(model))) * normal;
	gl_Position = projection * view * vertex_pos;
}